/* ========================================
 *
* File Name: MovingAverageFilter.c
*
* Description: 
*  Implements moving average filtering. 
*  Provides funtions 
*  1.To add new value into filter array
*  2.Test whether the filter inputs are stable
*  3.Provide filter output on demand.
* ========================================*/
#include "project.h"
#include "MovingAverageFilter.h"

#define MAF_STABLE_COUNTS 16
#define MAF_SAMPLE_COUNT 32

/*Moving Average Filter Array*/
int32 MAF_Samples[MAF_SAMPLE_COUNT];

/******************************************************************************
* Function Name: MAF_Init
*******************************************************************************
*
* Summary:
*  Initialized the Moving Average Filter Array to zero.
* 
*
******************************************************************************/
void MAF_Init(void)
{
    for (int index = 0; index < MAF_SAMPLE_COUNT; index++)
        MAF_Samples[index] = 0;
}

/******************************************************************************
* Function Name: MAF_Insert
*******************************************************************************
*
* Summary:
*  Insert a value into Moving Average Filter Array.
* 
*
******************************************************************************/
void MAF_Insert(int32 sample)
{
    for (int index = MAF_SAMPLE_COUNT - 1; index > 0; index--)
    {
        MAF_Samples[index] = MAF_Samples[index - 1];
    }
    MAF_Samples[0] = sample;
}

/******************************************************************************
* Function Name: MAF_IsStable
*******************************************************************************
*
* Summary:
*  Traverse through Moving Average Filter Array and confirm  whether the samples are stable.
*  The maximum difference between  any two samples in the array should be within 
*  MAF_STABLE_COUNTS.
*
******************************************************************************/
uint8 MAF_IsStable(void)
{
    uint8 result = 1u;
    int32 minimum, maximum;
    minimum = MAF_Samples[0];
    maximum = minimum;
    
    for (int index = 0; index < MAF_SAMPLE_COUNT; index++)
    {
        if (MAF_Samples[index] < minimum)
            minimum = MAF_Samples[index];
        if (MAF_Samples[index] > maximum)
            maximum = MAF_Samples[index];
    }
    if ((maximum - minimum) > MAF_STABLE_COUNTS)
    {
        result = 0u;
    }
    return (result);
}

/******************************************************************************
* Function Name: MAF_Average
*******************************************************************************
*
* Summary:
*  Returns the average of the Moving Average Filter Array.
*
******************************************************************************/
int32 MAF_Average(void)
{
    int32 result, sigma = 0;
    for (int index = 0; index < MAF_SAMPLE_COUNT; index++)
        sigma += MAF_Samples[index];
    result = sigma / MAF_SAMPLE_COUNT;
    if ((sigma % MAF_SAMPLE_COUNT) > (MAF_SAMPLE_COUNT / 2))
        result++;
    return (result);    
}

/******************************************************************************
* Function Name: MAF_Output
*******************************************************************************
*
* Summary:
*  Returns the average result if the output is stable. Otherwise returns the 
*  latest sample result. This makes the system responsive to changes.
*
******************************************************************************/
int32 MAF_Output(void)
{
    if (MAF_IsStable() == 1u)
        return (MAF_Average());
    else
        return (MAF_Samples[0]);
}

/* [] END OF FILE */
