/* ========================================
*
* File Name: main.c
*
* PSoC Creator Version 4.3
*
* Description: 
*  This example demonstrates how to interface a Bridge Sensor to PSoC5LP.
*
* Hardware Dependency: 
*  CY8CKIT-050
*
* ========================================
*/
#include "project.h"
#include "MovingAverageFilter.h"
#include <stdio.h>

/*Number of sensor samples in normal and inverted mode.
The measurement in either normal or inverted mode takes less than 9ms.
After measurement the mode is changed. After 1ms a new measurement is done.
There is no specific code for waiting for the AMux to switch between normal and
inverted modes */
#define SENSOR_AVERAGER_SAMPLE_COUNT 440

/*Delsig ADC is used in the continous sample mode. Initial samples are flushed out as the signal 
is multiplexed. ADC takes 3 samples to settle the to the new input. We discard 10 samples here. */
#define ADC_SAMPLES_TO_DISCARD 10

/*AMUX configuration to support Correlated Double Sampling*/
#define SENSOR_INPUT_NORMAL 0u
#define SENSOR_INPUT_INVERTED 1u

/*ADC Measurement configuration to support Correlated Double Sampling*/
#define MEASUREMENT_NORMAL 1u
#define MEASUREMENT_INVERTED 2u

/*ADC states on data acquisition*/
#define SENSOR_SAMPLE_READY 1u
#define SENSOR_SAMPLE_ACQUIRING 0u

uint8 Measurement_Phase = MEASUREMENT_NORMAL;

int32 sensor_averager_sigma = 0;
int32 sensor_averager_samples = 0;
int32 sensor_averager_output_phase1 = 0;
int32 sensor_averager_output_phase2 = 0;
int32 sensor_averager_output = 0;
uint8 sensor_sample_ready = SENSOR_SAMPLE_ACQUIRING;

/******************************************************************************
* Function Name: ADC_Sample_Handler
*******************************************************************************
*
* Summary:
*  Handles the Interrupt Service Routine for the Delta Sigma ADC.
*  Due to continous conversion using multiplexed input, fixed number of initial samples
*  are discarded. SENSOR_AVERAGER_SAMPLE_COUNT samples are averaged for both normal or invert modes.
*  The difference of  normal and inverted mode averages is the input signal in counts.
*
******************************************************************************/
CY_ISR(ADC_Sample_Handler)
{   
    /*Sample counts incremented*/
    sensor_averager_samples++;
    /*Fixed number of samples discarded on each measurement */
    if (sensor_averager_samples > ADC_SAMPLES_TO_DISCARD)
    {   
        /*Accumulate the ADC results*/
        sensor_averager_sigma += ADC_Sensor_GetResult32();
        
        if (sensor_averager_samples >= (ADC_SAMPLES_TO_DISCARD + SENSOR_AVERAGER_SAMPLE_COUNT))
        {   
            /*Stop ADC*/
            ADC_Sensor_StopConvert();
            if (Measurement_Phase == MEASUREMENT_NORMAL)
            {   
                /*ADC results are averaged*/
                sensor_averager_output_phase1 = sensor_averager_sigma / SENSOR_AVERAGER_SAMPLE_COUNT;
                /*Initiate inverted sampling*/
                Measurement_Phase = MEASUREMENT_INVERTED;
                /*Change AMux channel to inverted*/
                AMux_CDS_Select(SENSOR_INPUT_INVERTED);
            }
            else
            {
                sensor_averager_output_phase2 = sensor_averager_sigma / SENSOR_AVERAGER_SAMPLE_COUNT;
                /*CDS output*/
                sensor_averager_output = sensor_averager_output_phase1 - sensor_averager_output_phase2;
                Measurement_Phase = MEASUREMENT_NORMAL;
                /*Change AMux channel to normal*/
                AMux_CDS_Select(SENSOR_INPUT_NORMAL);
                sensor_sample_ready = SENSOR_SAMPLE_READY;
            }
            sensor_averager_sigma = 0;
            sensor_averager_samples = 0;
        }
    }
    /*Clear ADC interrupt*/
    ISR_ADC_Sample_ClearPending();
}

/******************************************************************************
* Function Name: Tick_Handler
*******************************************************************************
*
* Summary:
*  Handles the Interrupt Service Routine for the Tick Timer Component. Starts ADC Continuous conversion.
*  Amux channels switching will be settled by the time ISR is executed.
*  100ms is chosen the period of the timer since it offers 50/60Hz noise rejection.
*
******************************************************************************/
CY_ISR(Tick_Handler)
{
    sensor_averager_sigma = 0;
    sensor_averager_samples = 0;
    /*Start ADC continous conversion*/
    ADC_Sensor_StartConvert();
    /*Clear timer interrupt*/
    ISR_Tick_ClearPending();
}

/******************************************************************************
* Function Name: main
*******************************************************************************
*
* Summary:
*  The main function performs these functions:
*  1. Initializes and starts ADC, Timer, LCD, AMux and the Moving Average Filter.
*  2. Filter the ADC results using a moving average filter.
*  3. Prints the counts and weight into the LCD Display.
*
******************************************************************************/
int main(void)
{
    char lcd_line[17];
    
    /* Enable global interrupts. */
    CyGlobalIntEnable; 
    
    /*Initialize all the components. */
    MAF_Init();
    AMux_CDS_Start();
    AMux_CDS_Select(SENSOR_INPUT_NORMAL);
    ADC_Sensor_Start();
    LCD_Start();
    LCD_Init();
    LCD_Enable();
    LCD_PrintString("BridgeSensorAFE");
    LCD_Position(1, 0);
    LCD_PrintString(lcd_line);
    ISR_Tick_StartEx(Tick_Handler);
    ISR_ADC_Sample_StartEx(ADC_Sample_Handler);
    Timer_Tick_Start();

    for(;;)
    {
        if (sensor_sample_ready == SENSOR_SAMPLE_READY)
        {
            sensor_sample_ready = SENSOR_SAMPLE_ACQUIRING;
            MAF_Insert(sensor_averager_output);
            /*Print the result*/
            sprintf(lcd_line, "%8ld", MAF_Output());
            LCD_Position(1, 0);
            LCD_PrintString(lcd_line);
        }
    }
}

/* [] END OF FILE */
