/*******************************************************************************
* File Name: ADC_Sensor_PM.c
* Version 3.30
*
* Description:
*  This file provides the power manager source code to the API for the
*  Delta-Sigma ADC Component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2017, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "ADC_Sensor.h"

static ADC_Sensor_BACKUP_STRUCT ADC_Sensor_backup =
{
    ADC_Sensor_DISABLED,
    ADC_Sensor_CFG1_DEC_CR
};


/*******************************************************************************
* Function Name: ADC_Sensor_SaveConfig
********************************************************************************
*
* Summary:
*  Save the register configuration which are not retention.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  ADC_Sensor_backup:  This global structure variable is used to store
*  configuration registers which are non retention whenever user wants to go
*  sleep mode by calling Sleep() API.
*
*******************************************************************************/
void ADC_Sensor_SaveConfig(void) 
{
    ADC_Sensor_backup.deccr = ADC_Sensor_DEC_CR_REG;
}


/*******************************************************************************
* Function Name: ADC_Sensor_RestoreConfig
********************************************************************************
*
* Summary:
*  Restore the register configurations which are not retention.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  ADC_Sensor_backup:  This global structure variable is used to restore
*  configuration registers which are non retention whenever user wants to switch
*  to active power mode by calling Wakeup() API.
*
*******************************************************************************/
void ADC_Sensor_RestoreConfig(void) 
{
    ADC_Sensor_DEC_CR_REG = ADC_Sensor_backup.deccr;
}


/*******************************************************************************
* Function Name: ADC_Sensor_Sleep
********************************************************************************
*
* Summary:
*  Stops the operation of the block and saves the user configuration.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  ADC_Sensor_backup:  The structure field 'enableState' is modified
*  depending on the enable state of the block before entering to sleep mode.
*
*******************************************************************************/
void ADC_Sensor_Sleep(void) 
{
    /* Save ADC enable state */
    if((ADC_Sensor_ACT_PWR_DEC_EN == (ADC_Sensor_PWRMGR_DEC_REG & ADC_Sensor_ACT_PWR_DEC_EN)) &&
       (ADC_Sensor_ACT_PWR_DSM_EN == (ADC_Sensor_PWRMGR_DSM_REG & ADC_Sensor_ACT_PWR_DSM_EN)))
    {
        /* Component is enabled */
        ADC_Sensor_backup.enableState = ADC_Sensor_ENABLED;
        if((ADC_Sensor_DEC_CR_REG & ADC_Sensor_DEC_START_CONV) != 0u)
        {   
            /* Conversion is started */
            ADC_Sensor_backup.enableState |= ADC_Sensor_STARTED;
        }
		
        /* Stop the configuration */
        ADC_Sensor_Stop();
    }
    else
    {
        /* Component is disabled */
        ADC_Sensor_backup.enableState = ADC_Sensor_DISABLED;
    }

    /* Save the user configuration */
    ADC_Sensor_SaveConfig();
}


/*******************************************************************************
* Function Name: ADC_Sensor_Wakeup
********************************************************************************
*
* Summary:
*  Restores the user configuration and enables the power to the block.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  ADC_Sensor_backup:  The structure field 'enableState' is used to
*  restore the enable state of block after wakeup from sleep mode.
*
*******************************************************************************/
void ADC_Sensor_Wakeup(void) 
{
    /* Restore the configuration */
    ADC_Sensor_RestoreConfig();

    /* Enables the component operation */
    if(ADC_Sensor_backup.enableState != ADC_Sensor_DISABLED)
    {
        ADC_Sensor_Enable();
        if((ADC_Sensor_backup.enableState & ADC_Sensor_STARTED) != 0u)
        {
            ADC_Sensor_StartConvert();
        }
    } /* Do nothing if component was disable before */
}


/* [] END OF FILE */
