/* ========================================
*
* File Name: main.c
*
* PSoC Creator Version 4.3
*
* Description: 
*  This example demonstrates how to design a weighing scale using a load cell and PSoC5LP.
*
* Hardware Dependency: 
*  CY8CKIT-050
*
* ========================================
*/
#include "project.h"
#include "MovingAverageFilter.h"
#include <stdio.h>

/* 2,000gms and 0.5 gms resolution gives 4,000 counts */
#define FULL_SCALE_DISPLAY_COUNTS 4000

/* The actual sensor counts which corresponds to 2,000gms*/
/* This can also be used for calibration*/
#define FULL_SCALE_SENSOR_COUNTS 8000
#define COUNTS_TO_WEIGHT 0.5

/*Number of sensor samples in normal and inverted mode.
The measurement in either normal or inverted mode takes less than 9ms.
After measurement the mode is changed. After 1ms a new measurement is done.
There is no specific code for waiting for the AMux to switch between normal and
inverted modes */
#define SENSOR_AVERAGER_SAMPLE_COUNT 440

/*Delsig ADC is used in the continous sample mode. Initial samples are flushed out as the signal 
is multiplexed. ADC takes 3 samples to settle the to the new input. We discard 10 samples here. */
#define ADC_SAMPLES_TO_DISCARD 10

/*AMUX configuration to support Correlated Double Sampling*/
#define SENSOR_INPUT_NORMAL 0u
#define SENSOR_INPUT_INVERTED 1u

/*ADC Measurement configuration to support Correlated Double Sampling*/
#define MEASUREMENT_NORMAL 1u
#define MEASUREMENT_INVERTED 2u

/*ADC states on data acquisition*/
#define SENSOR_SAMPLE_READY 1u
#define SENSOR_SAMPLE_ACQUIRING 0u

/*Scale state to determine power-on to update zero weight*/
#define SCALE_POWERING_ON 1u
#define SCALE_POWERED_ON 0u

/*Switch debouncing for zero switch */
#define SW_DEBOUNCE_END 5u
#define SW_DEBOUNCE_START 0u

/*Moving Average Filter output is stable*/
#define MAF_STABLE 1u

uint8 Measurement_Phase = MEASUREMENT_NORMAL;

int32 sensor_averager_sigma = 0;
int32 sensor_averager_samples = 0;
int32 sensor_averager_output_phase1 = 0;
int32 sensor_averager_output_phase2 = 0;
int32 sensor_averager_output = 0;
uint8 sensor_sample_ready = SENSOR_SAMPLE_ACQUIRING;

uint8 SW_Zero_Debounce = SW_DEBOUNCE_START;

uint8 scale_powered_on = SCALE_POWERING_ON;
int32 scale_zero_counts;
int32 scale_value_counts;
int32 scale_display_counts;

/******************************************************************************
* Function Name: ADC_Sample_Handler
*******************************************************************************
*
* Summary:
*  Handles the Interrupt Service Routine for the Delta Sigma ADC.
*  Due to continous conversion using multiplexed input, fixed number of initial samples
*  are discarded. SENSOR_AVERAGER_SAMPLE_COUNT samples are averaged for both normal or invert modes.
*  The difference of  normal and inverted mode averages is the weight in counts.
*
******************************************************************************/
CY_ISR(ADC_Sample_Handler)
{   
    /*Sample counts incremented*/
    sensor_averager_samples++;
    /*Fixed number of samples discarded on each cycle*/
    if (sensor_averager_samples > ADC_SAMPLES_TO_DISCARD)
    {   
        /*Accumulate the ADC results*/
        sensor_averager_sigma += ADC_Sensor_GetResult32();
        
        if (sensor_averager_samples >= (ADC_SAMPLES_TO_DISCARD + SENSOR_AVERAGER_SAMPLE_COUNT))
        {   /*Stop ADC continous conversion*/
            ADC_Sensor_StopConvert();
            if (Measurement_Phase == MEASUREMENT_NORMAL)
            {   
                /*ADC results are averaged*/
                sensor_averager_output_phase1 = sensor_averager_sigma / SENSOR_AVERAGER_SAMPLE_COUNT;
                /*Initiate inverted sampling*/
                Measurement_Phase = MEASUREMENT_INVERTED;
                /*Change AMux channel to inverted*/
                AMux_CDS_Select(SENSOR_INPUT_INVERTED);
            }
            else
            {
                sensor_averager_output_phase2 = sensor_averager_sigma / SENSOR_AVERAGER_SAMPLE_COUNT;
                /*CDS output*/
                sensor_averager_output = sensor_averager_output_phase1 - sensor_averager_output_phase2;
                Measurement_Phase = MEASUREMENT_NORMAL;
                /*Change AMux channel to normal*/
                AMux_CDS_Select(SENSOR_INPUT_NORMAL);
                sensor_sample_ready = SENSOR_SAMPLE_READY;
            }
            sensor_averager_sigma = 0;
            sensor_averager_samples = 0;
        }
    }
    /*Clear ADC interrupt*/
    ISR_ADC_Sample_ClearPending();
}

/******************************************************************************
* Function Name: Tick_Handler
*******************************************************************************
*
* Summary:
*  Handles the Interrupt Service Routine for the Tick Timer Component. Starts ADC Continuous conversion.
*  Amux channels switching will be settled by the time ISR is executed.
*  100ms is chosen the period of the timer since it offers 50/60Hz noise rejection.
*
*  Update the zero counts on zero switch press. 
******************************************************************************/
CY_ISR(Tick_Handler)
{
    sensor_averager_sigma = 0;
    sensor_averager_samples = 0;
    /*Start ADC continous conversion*/
    ADC_Sensor_StartConvert();
    
    if (!Pin_SW_Zero_Read())
    {
        /* Switch kept pressed for */
        SW_Zero_Debounce++;
        /* Is switch pressed long enough */
        if (SW_Zero_Debounce >= SW_DEBOUNCE_END)
        {
            scale_zero_counts = MAF_Output();
        }
    }
    else
    {
        SW_Zero_Debounce = SW_DEBOUNCE_START;
    }    
    /*Clear timer interrupt*/
    ISR_Tick_ClearPending();
}

/******************************************************************************
* Function Name: main
*******************************************************************************
*
* Summary:
*  The main function performs these functions:
*  1. Initializes and starts ADC, Timer, LCD, AMux and the Moving Average Filter.
*  2. Initializes the zero counts at power-on.
*  3. Filter the ADC results using a moving average filter.
*  4. Prints the counts and weight into the LCD Display.
*
******************************************************************************/
int main(void)
{
    char lcd_line[17];
    double weight_value;
    
     /* Enable global interrupts. */
    CyGlobalIntEnable;

    /*Initialize all the components*/
    MAF_Init();
    AMux_CDS_Start();
    AMux_CDS_Select(0);
    ADC_Sensor_Start();
    LCD_Start();
    LCD_Init();
    LCD_Enable();
    LCD_PrintString("Weighing Scale");
    LCD_Position(1, 0);
    LCD_PrintString(lcd_line);
    ISR_Tick_StartEx(Tick_Handler);
    ISR_ADC_Sample_StartEx(ADC_Sample_Handler);
    Timer_Tick_Start();

    for(;;)
    {   
        /*If sample results are available*/
        if (sensor_sample_ready == SENSOR_SAMPLE_READY)
        {
            sensor_sample_ready = SENSOR_SAMPLE_ACQUIRING;
            MAF_Insert(sensor_averager_output);
            /*Update zero weight if it is power-on*/
            if (scale_powered_on == SCALE_POWERING_ON)
            {
                /*Check if filter output is stable*/
                if (MAF_IsStable() == MAF_STABLE)
                {
                    scale_zero_counts = MAF_Output();
                    scale_powered_on = SCALE_POWERED_ON;
                }
            }
            else
            {   
                /*Subract the zero counts*/
                scale_value_counts = MAF_Output() - scale_zero_counts;
                /*Convert counts to weight*/
                scale_display_counts = (scale_value_counts * FULL_SCALE_DISPLAY_COUNTS) / FULL_SCALE_SENSOR_COUNTS;
                weight_value = scale_display_counts * COUNTS_TO_WEIGHT;
                /*Print the conversion results*/
                sprintf(lcd_line, "%6ld %7.1f", scale_value_counts, weight_value);
                LCD_Position(1, 0);
                LCD_PrintString(lcd_line);
            }
            
        }
    }
}

/* [] END OF FILE */
